from testutils import change_workdir
import subprocess
import sys
import time
import hashlib
import shutil
import os
from conftest import ASSETS_DIR


def test_watch(tmp_path):
    def copy_image(name):
        shutil.copy(str(os.path.join(ASSETS_DIR, name)), str(tmp_path.joinpath("test.jpeg")))

    def write(content):
        tmp_path.joinpath("slides.py").write_text(
            f"""
from nelsie import SlideDeck

deck = SlideDeck()

@deck.slide()
def test(slide):
    {content}

deck.render("out.pdf")"""
        )

    def make_hash():
        with open("out.pdf", "rb") as f:
            return hashlib.md5(f.read())

    with change_workdir(tmp_path):
        copy_image("testimg.jpeg")
        write("slide.box(width=100, height=100, bg_color='green')")
        p = subprocess.Popen([sys.executable, "-m", "nelsie", "watch", "slides.py"])
        time.sleep(1.0)

        hash1 = make_hash()
        assert p.poll() is None

        copy_image("testimg.png")
        time.sleep(1.0)
        hash2 = make_hash()
        assert hash1.hexdigest() == hash2.hexdigest()

        write("slide.image('test.jpeg', width=100, height=100)")
        time.sleep(1.0)
        hash3 = make_hash()
        assert hash1.hexdigest() != hash3.hexdigest()

        copy_image("testimg.jpeg")
        time.sleep(1.0)
        hash4 = make_hash()
        assert hash3.hexdigest() != hash4.hexdigest()

        assert p.poll() is None
        p.kill()
        p.wait()



from .basictypes import AlignItems, AlignContent, FlexWrap, Stroke, TextAlign
from .insteps import InSteps
from .shapes import Arrow, Path
from .slidedeck import Resources, SlideDeck, Slide
from .textstyle import FontStretch, TextStyle
from .box import Box, BoxBuilder

__all__ = [
    "SlideDeck",
    "InSteps",
    "Path",
    "Stroke",
    "TextStyle",
    "Resources",
    "FontStretch",
    "TextAlign",
    "FlexWrap",
    "AlignItems",
    "AlignContent",
    "Arrow",
    "BoxBuilder",
    "Box",
    "Slide",
]



from testutils import check


@check()
def test_render_raster_image_native_size(deck):
    slide = deck.new_slide(width=400, height=400)
    slide.image("testimg.png")
    slide.image("testimg.jpeg")


@check(n_slides=5)
def test_render_raster_image_forced_size(deck):
    slide = deck.new_slide(width=420, height=400)
    slide.image("testimg.png", width="100%", height="100%")

    slide = deck.new_slide(width=400, height=400)
    slide.image("testimg.png", width=80, bg_color="gray")
    slide.image("testimg.jpeg", width=80, bg_color="blue")

    slide = deck.new_slide(width=400, height=400)
    slide.image("testimg.png", height=80, bg_color="gray")
    slide.image("testimg.jpeg", height=80, bg_color="blue")

    slide = deck.new_slide(width=400, height=400)
    slide.image("testimg.png", width="80%", bg_color="gray")

    slide = deck.new_slide(width=400, height=400)
    slide.image("testimg.png", height="80%", bg_color="gray")


@check(n_slides=4)
def test_render_svg_image_steps(deck):
    slide = deck.new_slide(width=420, height=400)
    slide.image("test.svg", width="90%")


@check(n_slides=1)
def test_render_svg_image_no_steps(deck):
    slide = deck.new_slide(width=420, height=400)
    slide.image("test.svg", width="90%", enable_steps=False)


@check(n_slides=6)
def test_render_svg_image_shift(deck):
    slide = deck.new_slide(width=100, height=100)
    box = slide.box(width="90%", height="80%", bg_color="gray")
    slide.insert_step(2)
    slide.insert_step(3)
    box.image("test.svg", width="80%", shift_steps=2)


@check(n_slides=3)
def test_render_ora_image_steps(deck):
    slide = deck.new_slide(width=250, height=250)
    box = slide.box(bg_color="gray")
    box.image("test.ora")


@check(n_slides=1)
def test_render_ora_image_no_steps(deck):
    slide = deck.new_slide(width=250, height=250)
    box = slide.box(bg_color="gray")
    box.image("test.ora", enable_steps=False)


@check(n_slides=1)
def test_render_ora_image_scale(deck):
    slide = deck.new_slide(width=400, height=400)
    slide.image("test.ora", width="80%", enable_steps=False, bg_color="gray")
    slide.image("test.ora", height="20%", enable_steps=False, bg_color="gray")


@check()
def test_svg_image_dtd(deck):
    slide = deck.new_slide(width=40, height=40)
    slide.image("knight_with_dtd.svg")



from testutils import check

from nelsie.layoutexpr import LayoutExpr


@check()
def test_fix_sizes(deck):
    slide = deck.new_slide()
    slide.box(width="100", height=300, bg_color="red")
    slide.box(width="100%", height="20", bg_color="blue")
    slide.box(width="50%", height="20%", bg_color="green")


@check(n_slides=4)
def test_layout_directions(deck):
    def make_slide(box):
        box.box(width=100, height=100, bg_color="red")
        box.box(width=200, height=200, bg_color="green")
        box.box(width=300, height=300, bg_color="blue")

    make_slide(deck.new_slide().box(width="100%", height="100%", row=False, reverse=False))
    make_slide(deck.new_slide().box(width="100%", height="100%", row=False, reverse=True))
    make_slide(deck.new_slide().box(width="100%", height="100%", row=True, reverse=False))
    make_slide(deck.new_slide().box(width="100%", height="100%", row=True, reverse=True))


@check(n_slides=5)
def test_layout_position(deck):
    def make_test_slide(row=False, **kwargs):
        slide = deck.new_slide(width=400, height=300)
        b = slide.box(row=row, width="50%", height="50%", bg_color="gray")
        b.box(width=20, height=20, bg_color="green")
        b2 = b.box(width=20, height=20, bg_color="red", **kwargs)
        b2.box(width="70%", height="70%", bg_color="orange")
        b.box(width=20, height=20, bg_color="green")

    # Slide 0 - both position defined
    make_test_slide(x=40, y=130)

    # Slide 1 - only x defined
    make_test_slide(x=40)

    # Slide 2 - only y defined
    make_test_slide(y=130)

    # Slide 3 - only x defined (row)
    make_test_slide(x=40, row=True)

    # Slide 4 - only y defined (row)
    make_test_slide(y=130, row=True)


@check()
def test_layout_position_string(deck):
    slide = deck.new_slide(width=400, height=300)
    slide.box(x="30", y="30%", width="50", height="50", bg_color="red")
    slide.box(x=30, y="80%", width="50", height="50", bg_color="blue")


@check()
def test_z_level(deck):
    slide = deck.new_slide(width=100, height=100)
    slide.box(x=0, y=0, width=75, height=75, z_level=1, bg_color="red")
    slide.box(x=25, y=25, width=75, height=75, bg_color="green")


@check()
def test_layout_padding(deck):
    slide = deck.new_slide(width=150, height=150)
    b = slide.box(bg_color="gray", p_left=20)
    b.box(width=15, height=15, bg_color="red")
    b = slide.box(bg_color="gray", p_right=20)
    b.box(width=15, height=15, bg_color="blue")
    b = slide.box(bg_color="gray", p_top=20)
    b.box(width=15, height=15, bg_color="red")
    b = slide.box(bg_color="gray", p_bottom=20)
    b.box(width=15, height=15, bg_color="blue")


@check()
def test_layout_margin(deck):
    slide = deck.new_slide(width=150, height=150)
    b = slide.box(bg_color="gray", m_left=20)
    b.box(width=15, height=15, bg_color="red")
    b = slide.box(bg_color="gray", m_right=20)
    b.box(width=15, height=15, bg_color="blue")
    b = slide.box(bg_color="gray", m_top=20)
    b.box(width=15, height=15, bg_color="red")
    b = slide.box(bg_color="gray", m_bottom=20)
    b.box(width=15, height=15, bg_color="blue")


@check()
def test_layout_flex_grow(deck):
    slide = deck.new_slide(width=150, height=150)
    b = slide.box(height="100%", bg_color="gray")
    b.box(width=30, height=15, bg_color="orange")
    b.box(width=30, flex_grow=1, bg_color="red")
    b.box(width=30, flex_grow=2, bg_color="blue")
    b.box(width=30, height=5, bg_color="#ff00ff")


@check()
def test_layout_flex_wrap(deck):
    slide = deck.new_slide(width=150, height=150)
    b = slide.box(row=True, height="100%", width="100%", bg_color="gray", flex_wrap="wrap")
    b.box(width=40, height=30, bg_color="orange")
    b.box(width=40, height=30, bg_color="blue")
    b.box(width=40, height=30, bg_color="orange")
    b.box(width=40, height=30, bg_color="blue")
    b.box(width=40, height=30, bg_color="orange")
    b.box(width=40, height=30, bg_color="blue")


@check(n_slides=4)
def test_layout_justify_content(deck):
    slide = deck.new_slide(width=50, height=150)
    b = slide.box(height="100%", width="100%", bg_color="gray", justify_content="start")
    b.box(width=20, height=30, bg_color="red")
    b.box(width=20, height=30, bg_color="green")
    b.box(width=20, height=30, bg_color="blue")

    slide = deck.new_slide(width=50, height=150)
    b = slide.box(height="100%", width="100%", bg_color="gray", justify_content="end")
    b.box(width=20, height=30, bg_color="red")
    b.box(width=20, height=30, bg_color="green")
    b.box(width=20, height=30, bg_color="blue")

    slide = deck.new_slide(width=50, height=150)
    b = slide.box(
        height="100%",
        width="100%",
        bg_color="gray",
        justify_content="space-evenly",
    )
    b.box(width=20, height=30, bg_color="red")
    b.box(width=20, height=30, bg_color="green")
    b.box(width=20, height=30, bg_color="blue")

    slide = deck.new_slide(width=50, height=150)
    b = slide.box(
        height="100%",
        width="100%",
        bg_color="gray",
        justify_content="space-around",
    )
    b.box(width=20, height=30, bg_color="red")
    b.box(width=20, height=30, bg_color="green")
    b.box(width=20, height=30, bg_color="blue")


@check(n_slides=3)
def test_layout_gap(deck):
    slide = deck.new_slide(width=50, height=150)
    b = slide.box(
        height="100%",
        width="100%",
        bg_color="gray",
        justify_content="start",
        gap=(0, 10),
    )
    b.box(width=20, height=30, bg_color="red")
    b.box(width=20, height=30, bg_color="green")
    b.box(width=20, height=30, bg_color="blue")

    slide = deck.new_slide(width=50, height=150)
    b = slide.box(
        height="100%",
        width="100%",
        bg_color="gray",
        justify_content="start",
        gap=(0, "15%"),
    )
    b.box(width=20, height=30, bg_color="red")
    b.box(width=20, height=30, bg_color="green")
    b.box(width=20, height=30, bg_color="blue")

    slide = deck.new_slide(width=150, height=50)
    b = slide.box(
        row=True,
        height="100%",
        width="100%",
        bg_color="gray",
        justify_content="start",
        gap=(30, 20),
    )
    b.box(width=20, height=30, bg_color="red")
    b.box(width=20, height=30, bg_color="green")
    b.box(width=20, height=30, bg_color="blue")


def test_layout_expr():
    e = LayoutExpr.x(123) + 10
    assert e._expr == ("sum", ("x", 123), 10)
    e2 = e - 20.0
    assert e2._expr == ("sum", ("x", 123), 10, -20.0)


@check()
def test_expr_x_y_weight_height(deck):
    slide = deck.new_slide(width=200, height=80)
    box = slide.box(width=180, height=40, bg_color="green")
    slide.box(
        x=box.x(0.33),
        y=box.y(),
        width=box.width(0.33),
        height=box.height(0.5),
        bg_color="blue",
    )


@check(n_slides=2)
def test_m_xy_p_xy(deck):
    slide = deck.new_slide(width=100, height=100)
    box = slide.box(bg_color="green")
    box.box(width=30, height=30, m_x=10, m_y=20, bg_color="red")

    slide = deck.new_slide(width=100, height=100)
    box = slide.box(bg_color="green", p_x=20, p_y=10)
    box.box(width=30, height=30, bg_color="red")



from nelsie import (
    Arrow,
    Path,
    SlideDeck,
    TextStyle,
    Resources,
    Stroke,
    InSteps,
)

resources = Resources()
resources.load_fonts_dir("karla_font")
deck = SlideDeck(resources=resources)

COLOR1 = "#333"
COLOR2 = "#ddccdd"

deck.update_style("default", TextStyle(font_family="Karla", color=COLOR1))
deck.set_style("highlight", TextStyle(color="#cc99cc", weight=800))
deck.set_style("title", TextStyle(size=64))


# First slide #############################################


@deck.slide()
def intro(slide):
    slide.image("../../docs/imgs/nelsie-logo.jpg", width="25%")
    title_box = slide.box(width="100%", p_top=20, p_bottom=30, m_top=50, m_bottom=50, bg_color=COLOR2)
    title_box.text("Nelsie", style=TextStyle(size=64, weight=800), m_bottom=15)
    title_box.text("Framework for Creating Slides", style=TextStyle(size=44))

    slide.set_style("email", slide.get_style("monospace").merge(TextStyle(size=18)))
    slide.text("Ada Böhm\n~email{ada@kreatrix.org}", m_bottom=50, align="center")


# Hello world #############################################


@deck.slide()
def hello_world(slide):
    slide.text("Hello world example", style="title", m_bottom=40)
    box = slide.box(p_left=20, p_right=20, p_top=20, p_bottom=20, bg_color="#eee")
    box.code(
        """
from nelsie import SlideDeck

deck = SlideDeck()

@deck.slide()
def hello_world(slide):
    slide.text("Hello world!")

deck.render("slides.pdf")
""",
        language="py",
    )


# Fragments ##########################################


@deck.slide()
def fragments(slide):
    slide.text("Nelsie supports ...", style="title")

    # A single slide may generate more pages in resulting pdf
    # Argument 'show' controls on which pages of the slides is the box shown.
    # Possible values:
    # "2" - Show only on the second page
    # "2+" - Show from the second page to the last page
    # "2-5" - Show on second page to 5th page
    # "2,3,4" - Show on pages 2, 3, and 4
    slide.box(show="2+").text("... fragment ...", style="title")
    slide.box(show="3+").text("... revealing.", style="title")


# InSteps ##########################################


@deck.slide()
def in_steps(slide):
    slide.text("Simple mechanism for complex slide changes", m_bottom=40)

    semaphore = slide.box(width=100, height=300, bg_color="gray")
    semaphore.box(
        y=InSteps({1: 20, 2: 110, 3: 210}),
        width=80,
        height=80,
        bg_color=InSteps({1: "red", 2: "orange", 3: "green"}),
    )


# Layers in SVGs and ORA images ##########################################


@deck.slide()
def images(slide):
    slide.text("Name-driven revealing layers in SVG and ORA images", m_bottom=40)
    slide.image("imgs/stepped_logo.ora", width="25%")
    slide.image("imgs/layers.svg", width="80%")


# Header ##########################################


@deck.slide()
def header_and_footer(slide):
    header = slide.box(
        row=True,
        width="100%",
        bg_color="#aa99aa",
        justify_content="end",
        p_top=25,
        p_bottom=25,
        p_right=20,
    )
    header.text("Header Demo", TextStyle(color="white"))
    slide.box(width="100%", height="10", bg_color="gray")

    slide.box(flex_grow=1).text("Headers & Titles", style="title")

    footer = slide.box(row=True, width="100%", height="50", bg_color=COLOR2)
    footer.box(flex_grow=1).text("Hello!", TextStyle(size=24, color="gray"))
    footer.box(flex_grow=1, height="100%", bg_color="#aa99aa").text("Footer", TextStyle(size=24))
    footer.box(flex_grow=1).text("Hello!", TextStyle(size=24, color="gray"))


# Syntax highlighting #################################

CODE_EXAMPLE = """
# This is comment
from nelsie import SlideDeck

deck = SlideDeck()

@deck.slide()
def hello_world(slide):
    slide.text("Hello world!")

deck.render("slides.pdf")
"""


@deck.slide()
def syntax_highlighting(slide):
    slide.text("Syntax highlighting", "title", m_bottom=50)

    slide.code(CODE_EXAMPLE, language="py")


# Line highlighting #################################


@deck.slide()
def line_highlighting(slide):
    slide.text("Line highlighting", "title", m_bottom=50)

    text = slide.code(CODE_EXAMPLE, language="py")

    text.line_box(6, bg_color="#bb99bb", z_level=-1)


# Pointing into text (1) #################################


@deck.slide()
def text_pointers1(slide):
    slide.text("Pointing into text", "title", m_bottom=50)

    text = slide.code(CODE_EXAMPLE, language="py")

    arrow = Arrow(size=20)
    slide.draw(
        Path(stroke=Stroke(color="orange", width=5), arrow_end=arrow)
        .move_to(text.line_x(3, 1) + 5, text.line_y(3, 0.5))
        .line_to(text.line_x(9, 1) + 50, text.line_y(3, 0.5))
        .line_to(text.line_x(9, 1) + 50, text.line_y(9, 0.5))
        .line_to(text.line_x(9, 1) + 5, text.line_y(9, 0.5))
    )


# Pointing into text (2) #################################


@deck.slide()
def text_pointers2(slide):
    slide.text("Pointing into text", "title", m_bottom=50)

    text = slide.code(CODE_EXAMPLE, language="py")

    comment = slide.box(
        x="70%",
        y="45%",
        p_left=30,
        p_right=30,
        p_top=30,
        p_bottom=30,
        bg_color="green",
        border_radius=8,
    )
    comment.text("Comment", style=TextStyle(color="white"))

    slide.draw(
        Path(fill_color="green")
        .move_to(text.line_x(5, 1.0) + 5, text.line_y(5, 0.5))
        .line_to(comment.x(0.1), comment.y(0.25))
        .line_to(comment.x(0.1), comment.y(0.75))
    )


# Pointing into text (3) #################################


@deck.slide()
def text_pointers3(slide):
    slide.text("Pointing into text", "title", m_bottom=50)

    text = slide.code(
        """
# This is comment
from nelsie import SlideDeck

deck = SlideDeck()

@deck.slide()
def hello_world(~1{slide}):
    ~2{slide}.text("Hello world!")

deck.render("slides.pdf")
""",
        language="py",
        parse_styles=True,
    )

    for anchor_id in 1, 2:
        text.text_anchor_box(anchor_id, bg_color="#aaa", z_level=-1)

    arrow = Arrow(size=15)
    slide.draw(
        [
            Path(stroke=Stroke(color="green", width=5), arrow_end=arrow)
            .move_to(text.text_anchor_x(2, 0) - 20, text.text_anchor_y(2, 1) + 20)
            .line_to(text.text_anchor_x(2, 0), text.text_anchor_y(2, 1)),
            Path(stroke=Stroke(color="green", width=5), arrow_end=arrow)
            .move_to(text.text_anchor_x(2, 1) + 20, text.text_anchor_y(2, 1) + 20)
            .line_to(text.text_anchor_x(2, 1), text.text_anchor_y(2, 1)),
        ]
    )


# Overwriting styles in syntax highlight #################################


@deck.slide()
def syntax_highlight_with_own_styles(slide):
    slide.text("Own styles in syntax highlight", "title", m_bottom=50)

    slide.set_style("grayout", TextStyle(color="gray", weight=400))
    slide.code(
        """
~grayout{# This is comment
from nelsie import SlideDeck

deck = SlideDeck()}

@deck.slide()
def hello_world(~1{slide}):
    ~2{slide}.text("Hello world!")

~grayout{deck.render("slides.pdf")}
""",
        language="py",
        parse_styles=True,
    )


# Console demo ############################################


@deck.slide()
def console_demo(slide):
    slide.set_style("shell", slide.get_style("code").merge(TextStyle(color="white", size=24)))
    slide.set_style("prompt", TextStyle(color="#aaaaff"))
    slide.set_style("cmd", TextStyle(color="yellow"))

    slide.text("Console demo", "title", m_bottom=30)

    # The 'console' is just text with a few styles
    # As we want to use "~" character in the text,
    # we are changing escape_char for styles from "~" to "!"
    console = slide.box(bg_color="black", p_left=20, p_right=20, p_top=10, p_bottom=10, border_radius=10)
    console.text(
        "!prompt{~/nelsie/example/bigdemo$} !cmd{ls}\n"
        "bigdemo.py  imgs  karla_font\n\n"
        "!prompt{~/nelsie/example/bigdemo$} !cmd{python3 bigdemo.py}\n",
        style="shell",
        style_delimiters="!{}",
    )


# Shapes ##########################################


@deck.slide()
def shapes(slide):
    slide.text("Shapes", "title", m_bottom=40)

    box = slide.box(width=700, height=100, m_bottom=60)
    rect = (
        Path(stroke=Stroke(color="red", width=5))
        .move_to(0, 0)
        .line_to(100, 0)
        .line_to(100, 100)
        .line_to(0, 100)
        .close()
    )
    triangle = Path(fill_color="green").move_to(200, 100).line_to(250, 0).line_to(300, 100)
    circle = Path.oval(400, 0, 500, 100, fill_color="blue")

    rounded_box = (
        Path(stroke=Stroke(color="orange", width=5))
        .move_to(650, 0)
        .quad_to(700, 0, 700, 50)
        .quad_to(700, 100, 650, 100)
        .quad_to(700, 100, 650, 100)
        .quad_to(600, 100, 600, 50)
        .line_to(650, 50)
    )
    box.draw([rect, triangle, rounded_box, circle])

    box = slide.box(width=700, height=70, m_bottom=60)
    box.draw(
        [
            Path(stroke=Stroke(color="black", width=10, dash_array=[10])).move_to(0, 0).line_to(700, 0),
            Path(stroke=Stroke(color="black", width=10, dash_array=[10, 20], dash_offset=15))
            .move_to(0, 30)
            .line_to(700, 30),
            Path(stroke=Stroke(color="black", width=10, dash_array=[30, 10, 5, 10])).move_to(0, 60).line_to(700, 60),
        ]
    )

    box = slide.box(width=700, height=220)
    arrow1 = Arrow(size=30)
    arrow2 = Arrow(size=30, angle=20)
    arrow3 = Arrow(size=30, inner_point=0.8)
    arrow4 = Arrow(size=30, inner_point=2.4)
    arrow5 = Arrow(size=30, stroke_width=5)

    box.draw(
        [
            Path(
                stroke=Stroke(color="black", width=5),
                arrow_start=arrow1,
                arrow_end=arrow1,
            )
            .move_to(0, 0)
            .line_to(700, 0),
            Path(
                stroke=Stroke(color="black", width=5),
                arrow_start=arrow2,
                arrow_end=arrow2,
            )
            .move_to(0, 50)
            .line_to(700, 50),
            Path(
                stroke=Stroke(color="black", width=5),
                arrow_start=arrow3,
                arrow_end=arrow3,
            )
            .move_to(0, 100)
            .line_to(700, 100),
            Path(
                stroke=Stroke(color="black", width=5),
                arrow_start=arrow4,
                arrow_end=arrow4,
            )
            .move_to(0, 150)
            .line_to(700, 150),
            Path(
                stroke=Stroke(color="black", width=5),
                arrow_start=arrow5,
                arrow_end=arrow5,
            )
            .move_to(0, 200)
            .line_to(700, 200),
        ]
    )


# Path demo ##########################################


@deck.slide()
def path_demo(slide):
    root = slide.box(
        x=250,
        y="50%",
        bg_color="#777",
        border_radius=10,
        p_left=20,
        p_right=20,
        p_top=20,
        p_bottom=20,
    )
    root.text("Root", style=TextStyle(color="white"))
    child1 = slide.box(
        x=650,
        y="20%",
        bg_color="#777",
        border_radius=10,
        p_left=20,
        p_right=20,
        p_top=20,
        p_bottom=20,
    )
    child1.text("Child 1", style=TextStyle(color="white"))
    child2 = slide.box(
        x=650,
        y="80%",
        bg_color="#777",
        border_radius=10,
        p_left=20,
        p_right=20,
        p_top=20,
        p_bottom=20,
    )
    child2.text("Child 2", style=TextStyle(color="white"))
    arrow = Arrow(20)
    x0, y0 = root.x(1), root.y(0.5)
    x1, y1 = child1.x(0), child1.y(0.5)
    x2, y2 = child2.x(0), child2.y(0.5)

    x1a, y1a = child1.x(1), child1.y(0.5)
    x1b, y1b = child1.x(0.5), child1.y(0)

    slide.draw(
        [
            Path(stroke=Stroke(color="#777", width=2), arrow_end=arrow)
            .move_to(x0, y0)
            .cubic_to(x0 + 300, y0, x1 - 300, y1, x1, y1),
            Path(stroke=Stroke(color="#777", width=2), arrow_end=arrow).move_to(x0, y0).quad_to(x2 - 100, y2, x2, y2),
            Path(stroke=Stroke(color="#777", width=2), arrow_end=arrow)
            .move_to(x1a, y1a)
            .quad_to(x1a + 50, y1a, x1a + 50, y1a - 50)
            .cubic_to(x1a + 50, y1a - 100, x1b, y1b - 100, x1b, y1b),
        ]
    )

    box = slide.box(x=680, y="50%")
    box.draw(
        Path(stroke=Stroke(color="#777", width=2), fill_color="orange")
        .move_to(40, 0)
        .line_to(80, 40)
        .line_to(40, 80)
        .line_to(0, 40)
        .close()
    )


# Chessboard demo ##########################################


@deck.slide()
def chess_board(slide):
    # Draw chessboard
    colors = [COLOR2, "#665566"]
    tiles = {}
    board = slide.box(width=500, height=500)
    for i in range(8):
        row = board.box(width="100%", flex_grow=1.0, row=True)
        for j in range(8):
            b = row.box(height="100%", flex_grow=1.0, bg_color=colors[(i + j) % 2])
            tiles[(j, i)] = b.overlay(z_level=0)

    # Draw arrow
    slide.overlay(show="1-3", z_level=1).draw(
        Path(stroke=Stroke(color="black", width=15), arrow_end=Arrow(30))
        .move_to(tiles[(3, 4)].x(0.5), tiles[(3, 4)].y(0.5))
        .line_to(tiles[(3, 2)].x(0.5), tiles[(3, 2)].y(0.5))
        .line_to(tiles[(4, 2)].x(0.5), tiles[(4, 2)].y(0.5))
    )

    # Draw knight
    tiles[(3, 4)].box(show="1", z_level=2).image("imgs/knight.svg")
    tiles[(3, 3)].box(show="2", z_level=2).image("imgs/knight.svg")
    tiles[(3, 2)].box(show="3", z_level=2).image("imgs/knight.svg")
    tiles[(4, 2)].box(show="4", z_level=2).image("imgs/knight.svg")


# Links


@deck.slide()
def links(slide):
    slide.text("Clicable links in PDF", TextStyle(size=80, underline=True), url="https://github.com/spirali/nelsie")


# Debugging frames ##########################################


@deck.slide(debug_layout=True)
def debugging_frames(slide):
    slide.image("../../docs/imgs/nelsie-logo.jpg", width="25%")
    title_box = slide.box(
        width="100%",
        p_top=20,
        p_bottom=30,
        m_top=50,
        m_bottom=50,
        bg_color=COLOR2,
        name="title box",
        debug_layout="black",
    )
    title_box.text("Nelsie", style=TextStyle(size=64, weight=800), m_bottom=15)
    title_box.text("Framework for Creating Slides", style=TextStyle(size=44))

    slide.set_style("email", slide.get_style("monospace").merge(TextStyle(size=18)))
    slide.text("Ada Böhm\n~email{ada@kreatrix.org}", m_bottom=50, align="center")

    slide.text(x="65%", y="20%", text="Debugging\nframes", style="title")


# FINAL RENDER

deck.render("bigdemo.pdf")



from testutils import check

from nelsie import InSteps, TextStyle


def test_step_values():
    def check(in_steps, values, n_steps):
        assert in_steps.in_step_values == values

    check(InSteps(["red", "red", "blue"]), {1: "red", 3: "blue"}, 3)

    check(
        InSteps({2: "black", 1: "orange", 4: "green"}),
        {2: "black", 1: "orange", 4: "green"},
        4,
    )


@check(n_slides=3)
def test_render_steps(deck):
    slide = deck.new_slide()
    slide.box(
        width=100,
        height=InSteps({1: "75%", 2: "25%"}),
        bg_color=InSteps(["red", "green", "blue"]),
    )


@check(n_slides=3)
def test_render_substeps(deck):
    slide = deck.new_slide(step_1=False)
    slide.box(
        width=100,
        height=InSteps({(2, 3, 1): "75%", (2, 3, 2): "25%"}),
        bg_color=InSteps({(2, 3, 1): "red", (2, 3, 2): "green", 4: "blue"}),
    )


@check(n_slides=4)
def test_show_steps(deck):
    slide = deck.new_slide(width=200, height=200)
    b = slide.box(show="2+", width=100, height=100, bg_color="red")
    b.box(show="1, 3", width=40, height=40, bg_color="blue")
    b.box(show="4", width=40, height=40, bg_color="green")


@check(n_slides=3)
def test_active_steps(deck):
    slide = deck.new_slide(width=200, height=200)
    slide.box(width=30, height=30, bg_color="red")
    slide.box(width=30, height=30, bg_color="green", active="2")
    slide.box(width=30, height=30, bg_color="orange", active=3)
    slide.box(width=30, height=30, bg_color="blue")


@check(n_slides=4)
def test_replace_steps(deck):
    slide = deck.new_slide(width=200, height=200)
    slide.box(
        replace_steps={1: 2, 3: 1},
        width=InSteps({1: "100", 2: "50", 3: "20"}),
        height=20,
        bg_color="green",
    )
    slide.image("test.svg", width="50%", replace_steps={1: 3, 2: 1})


def test_set_get_steps(deck):
    slide = deck.new_slide()
    assert slide.get_steps() == [(1,)]
    slide.box(width=InSteps({1: 100, 3: 200, 5: 600}))
    slide.box(width=InSteps({5: 600, (5, 3): 300}))
    assert slide.get_steps() == [(1,), (3,), (5,), (5, 3)]
    slide.insert_step(3)
    slide.insert_step((6, 2))
    assert slide.get_steps() == [(1,), (3,), (5,), (5, 3), (6, 2)]

    slide.remove_step(5)
    assert slide.get_steps() == [(1,), (3,), (5, 3), (6, 2)]

    slide.remove_steps_above((5, 3))
    assert slide.get_steps() == [(1,), (3,), (5, 3)]

    slide.remove_steps_below((5, 3))
    assert slide.get_steps() == [(5, 3)]


@check(n_slides=4)
def test_step_global_counter(deck):
    deck.set_style("default", TextStyle(size=12))
    deck.set_style("g", TextStyle(color="green"))
    deck.set_style("r", TextStyle(color="red"))
    slide = deck.new_slide(width=100, height=40)
    slide.text(
        "$(global_slide)/$(global_slides)  $(global_page)/$(global_pages)",
        parse_counters=True,
        bg_color="gray",
    )
    slide.insert_step(2)
    slide = deck.new_slide(width=400, height=100)
    slide.text(
        "$(global_slide)/$(global_slides)  $(global_page)/$(global_pages)",
        bg_color="gray",
    )
    slide = deck.new_slide(width=100, height=40)
    slide.text(
        "$(global_slide)/$(global_slides) ~g{!!!} ~r{$(global_page)}/$(global_pages) ~g{!!!}",
        parse_counters=True,
        bg_color="gray",
    )


@check(n_slides=8)
def test_step_other_counter(deck):
    def create_slide(counters=None, color="black"):
        slide = deck.new_slide(width=100, height=40, counters=counters)
        slide.text(
            "$(my_slide)/$(my_slides)  $(my_page)/$(my_pages)",
            style=TextStyle(color=color),
            parse_counters=True,
            bg_color="gray",
        )
        return slide

    deck.set_style("default", TextStyle(size=12))
    create_slide()
    create_slide(counters=["my"], color="red")
    create_slide(counters=["other"])
    slide = create_slide(counters=["my"], color="blue")
    slide.insert_step(2)
    slide.insert_step(3)
    slide = create_slide()
    slide.insert_step(2)


@check()
def test_step_invalid_counter(deck):
    deck.set_style("default", TextStyle(size=12))
    deck.set_style("g", TextStyle(color="green"))
    slide = deck.new_slide(width=100, height=40)
    slide.text("$(global_~g{page})", bg_color="gray", parse_counters=True)


@check(n_slides=4)
def test_show_next_last_keywords(deck):
    deck.set_style("default", TextStyle(size=12))
    slide = deck.new_slide(width=100, height=100)
    slide.box(show="last").text("Last")
    slide.box(show="next").text("Next")
    slide.box(show="last+").text("Last+")
    slide.box(show="next+").text("Next+")
    slide.box(show=3).text("Jump")
    slide.box(show="last").text("Last2")
    slide.box(show="next").text("Next2")
    print(slide.get_steps())


@check(n_slides=4)
def test_active_next_last_keywords(deck):
    deck.set_style("default", TextStyle(size=12))
    slide = deck.new_slide(width=100, height=100)
    slide.box(active="last").text("Last")
    slide.box(active="next").text("Next")
    slide.box(active="last+").text("Last+")
    slide.box(active="next+").text("Next+")
    slide.box(active=3).text("Jump")
    slide.box(active="last").text("Last2")
    slide.box(active="next").text("Next2")


@check(n_slides=8)
def test_subslides(deck):
    deck.set_style("default", TextStyle(size=8))

    def counters(parent):
        text = "$(global_slide)/$(global_slides) $(global_page)/$(global_pages)"
        parent.text(text, x=0, y=0, parse_counters=True, z_level=1)

    slide = deck.new_slide(width=40, height=70)
    counters(slide)
    slide.box(width=20, height=20, bg_color="red")
    slide.box(width=20, height=20, bg_color="orange", show="next+")
    slide.box(width=20, height=20, bg_color="green", show="next+")

    slide2 = slide.new_slide_at(3, width=40, height=40)
    counters(slide2)
    slide2.box(width=10, height=10, bg_color="blue")
    slide2.box(width=10, height=10, bg_color="blue", show="next+")

    slide3 = slide.new_slide_at(3, width=40, height=40)
    counters(slide3)
    slide3.box(width=10, height=10, bg_color="purple")

    slide4 = slide2.new_slide_at(3, width=40, height=40)
    counters(slide4)
    slide4.box(width=5, height=5, bg_color="gray")

    slide5 = slide.new_slide_at(6, width=40, height=40)
    counters(slide5)
    slide5.box(width=10, height=10, bg_color="black")


@check(n_slides=4)
def test_subslides_decorator(deck):
    deck.set_style("default", TextStyle(size=12))

    @deck.slide(width=100, height=50)
    def my_slide(slide):
        slide.box().text("One")
        slide.box(show="next+").text("Two")
        slide.box(show="next+").text("Three")

    @my_slide.slide_at(3, width=100, height=50)
    def inserted(slide):
        slide.text("Inserted")


@check(n_slides=1)
def test_invisible_steps(deck):
    slide = deck.new_slide(step_1=False)
    slide.insert_step(0)
    slide = deck.new_slide(width=20, height=20)
    slide.box(width=10, height=10, bg_color=InSteps({(0, 1): "green"}))


@check(n_slides=3)
def test_debug_steps(deck):
    slide = deck.new_slide(debug_steps=True, width=300, height=300)
    slide.text("Hello", show="2+")
    slide.text("World", show=(22, 3, 111))



from dataclasses import dataclass

from .basictypes import Stroke
from .layoutexpr import LayoutExpr

PathValue = int | float | LayoutExpr


@dataclass
class Arrow:
    """
    Represents an SVG arrow head. Can be attached to the start or end points of lines.
    """

    size: float = 10
    """Size of the arrow head in pixels."""

    angle: float = 40
    """Angle of the arrow head."""

    color: str | None = None
    """Color of arrow, if None color is taken from path"""

    stroke_width: float | None = None
    """If None then a filled arrow is drawn, if float then stroked arrow is drawn with the given stroke width"""

    inner_point: float | None = None
    """ Shape of the arrow head.

        * < 1.0 -> Sharper arrow.
        * = 1.0 -> Normal arrow.
        * > 1.0 -> Diamond shape arrow.
    """


class Path:
    def __init__(
        self,
        *,
        stroke: Stroke | None = None,
        fill_color: str | None = None,
        arrow_start: Arrow | None = None,
        arrow_end: Arrow | None = None,
    ):
        self.stroke = stroke
        self.fill_color = fill_color
        self.commands = []
        self.points = []
        self.arrow_start = arrow_start
        self.arrow_end = arrow_end

    @staticmethod
    def oval(
        x1: PathValue,
        y1: PathValue,
        x2: PathValue,
        y2: PathValue,
        *,
        stroke: Stroke | None = None,
        fill_color: str | None = None,
    ):
        path = Path(stroke=stroke, fill_color=fill_color)
        path.commands.append("oval")
        path.points = [x1, y1, x2, y2]
        return path

    def close(self):
        self.commands.append("close")
        return self

    def move_to(self, x: PathValue, y: PathValue):
        self.commands.append("move")
        self.points.append(x)
        self.points.append(y)
        return self

    def line_to(self, x: PathValue, y: PathValue):
        self.commands.append("line")
        self.points.append(x)
        self.points.append(y)
        return self

    def quad_to(self, x1: PathValue, y1: PathValue, x: PathValue, y: PathValue):
        self.commands.append("quad")
        self.points.append(x1)
        self.points.append(y1)
        self.points.append(x)
        self.points.append(y)
        return self

    def cubic_to(
        self,
        x1: PathValue,
        y1: PathValue,
        x2: PathValue,
        y2: PathValue,
        x: PathValue,
        y: PathValue,
    ):
        self.commands.append("cubic")
        self.points.append(x1)
        self.points.append(y1)
        self.points.append(x2)
        self.points.append(y2)
        self.points.append(x)
        self.points.append(y)
        return self



from dataclasses import asdict

import pytest
from testutils import check

import nelsie
from nelsie import FontStretch, InSteps, Stroke, TextStyle


def test_text_update():
    s1 = TextStyle(color="green", size=123)
    s2 = TextStyle(size=321, line_spacing=1.5)
    s3 = s1.merge(s2)
    assert s3.color == "green"
    assert s3.size == 321
    assert s3.line_spacing == 1.5


def test_text_invalid_font(deck):
    s1 = TextStyle(font_family="Nonexisting font")
    with pytest.raises(Exception, match="Font 'Nonexisting font' not found"):
        deck.new_slide().text("Hello", style=s1)


@check(n_slides=4)
def test_render_text_basic(deck):
    deck.set_style("highlight", TextStyle(color="orange"))
    slide = deck.new_slide()
    slide.set_style("small", TextStyle(size=8))
    slide.box(bg_color="#f88").text(
        "Hello ~highlight{world! ~small{this is small}}. End of text.\nNew line\nThird line"
    )
    slide = deck.new_slide()
    slide.box(bg_color="#f88").text("A\n\nBB")

    slide = deck.new_slide()
    slide.box(bg_color="#f88").text("\nLines up & below\n\n\n", strip=False)

    slide = deck.new_slide()
    slide.set_style("big", TextStyle(size=64))
    slide.box(bg_color="#f88").text("Now follows: ~big{Big text}\nNext line\nNext line\nNext line")


@check(n_slides=2)
def test_render_text_steps(deck):
    deck.set_style("highlight", TextStyle(color="orange"))
    slide = deck.new_slide(width=300, height=100)
    slide.set_style(
        "my_style",
        InSteps({1: TextStyle(color="green"), 2: TextStyle(color="orange", size=64)}),
    )
    #    slide.box(bg_color="#f88").text("Say ~my_style{hello}!")
    slide.box().text("Say ~my_style{hello}!")


@check()
def test_render_text_unicode(deck):
    deck.set_style("x", TextStyle(color="yellow"))
    slide = deck.new_slide()
    slide.box(bg_color="#f88").text("Příliš žluťoučký ~x{kůň} úpěl ďábelské ódy\n>>>y̆<<<")


def test_set_invalid_font(deck):
    with pytest.raises(Exception, match="Font 'NON-existent-fnt' not found."):
        deck.set_style("my_style", TextStyle(font_family="NON-existent-fnt"))


def test_set_get_styles_deck(deck):
    s = deck.get_style("default")
    for key, value in asdict(s).items():
        assert value is not None

    assert s.font_family in ("DejaVu Sans", "Arial")
    assert s.color == "#000000"
    assert s.size == pytest.approx(32.0)
    assert s.line_spacing == pytest.approx(1.2)

    with pytest.raises(Exception, match="Style 'big' not found"):
        deck.get_style("big")

    deck.set_style("big", TextStyle(size=120.0))
    s = deck.get_style("big")
    for key, value in asdict(s).items():
        if key != "size":
            assert value is None
    assert s.size == pytest.approx(120.0)


# On windows, underline is drawn in slightly different place, I do not know why
@check(windows_threshold=1800)
def test_text_lines(deck):
    slide = deck.new_slide(width=200, height=250)
    slide.text("Test text", TextStyle(underline=True), m_bottom=10)
    slide.text("Test text", TextStyle(overline=True), m_bottom=10)
    slide.text("Test text", TextStyle(line_through=True), m_bottom=10)
    slide.text("Test text", TextStyle(underline=True, overline=True, line_through=True))


def test_set_get_styles_box(deck):
    slide = deck.new_slide()
    slide.set_style("one", TextStyle(color="red"))
    slide.set_style("two", TextStyle(color="green"))
    b = slide.box()
    b2 = b.box()
    b2.set_style("one", TextStyle(color="blue"))
    b2.set_style("three", TextStyle(size=321))
    b2.set_style("four", InSteps({1: TextStyle(size=100), 4: TextStyle(size=40)}))
    b3 = b2.box()
    b3.set_style("default", TextStyle(line_spacing=1.0))

    with pytest.raises(Exception, match="Style 'three' not found"):
        deck.get_style("three")
    with pytest.raises(Exception, match="Style 'three' not found"):
        b.get_style("three")

    s = b.get_style("one")
    assert s == TextStyle(color="#ff0000")

    s = b3.get_style("one")
    assert s == TextStyle(color="#0000ff")

    s = b3.get_style("three")
    assert s.size == pytest.approx(321.0)

    s = b3.get_style("default")
    assert s.line_spacing == 1.0
    assert s.color == "#000000"


@check()
def test_text_color_opacity(deck):
    slide = deck.new_slide(width=220, height=50)
    slide.set_style("one", TextStyle(color="#ff00ff50"))
    slide.box(x=0, y=0, width=60, height="100%", bg_color="green")
    slide.box(x=60, y=0, width="30%", height="100%", bg_color="blue")
    slide.text("Opacity test", style="one")
    assert slide.get_style("one").color == "#ff00ff50"


@check()
def test_text_styling(deck):
    slide = deck.new_slide(width=220, height=150)
    slide.text("Italic", style=TextStyle(italic=True))
    slide.text("Bold", TextStyle(weight=700))
    slide.text("Hi are you?", TextStyle(stretch=FontStretch.UltraCondensed, size=10))
    slide.text("Hi are you?", TextStyle(stretch=FontStretch.UltraExpanded, size=10))


def test_text_style_get_stretch(deck):
    slide = deck.new_slide(width=220, height=150)
    slide.set_style("test", TextStyle(stretch=FontStretch.Expanded))
    assert isinstance(slide.get_style("test").stretch, nelsie.FontStretch)
    assert slide.get_style("default").stretch == FontStretch.Normal
    assert slide.get_style("test").stretch == FontStretch.Expanded


@check()
def test_text_stroke(deck):
    slide = deck.new_slide(width=150, height=100, bg_color="orange")
    slide.text("Text 1", style=TextStyle(stroke=Stroke("green"), color="#909090"))
    slide.text(
        "Text 2",
        style=TextStyle(stroke=Stroke("blue", dash_array=[5, 2], width=0.2), color="empty"),
    )


def test_text_style_get_stroke(deck):
    slide = deck.new_slide(width=220, height=150)
    slide.set_style(
        "test",
        TextStyle(stroke=Stroke("green", dash_array=[5, 2], width=10, dash_offset=2)),
    )
    assert slide.get_style("default").stroke == "empty"
    assert slide.get_style("test").stroke == Stroke("#008000", dash_array=[5, 2], width=10, dash_offset=2)


@check()
def test_text_monospace(deck):
    slide = deck.new_slide(width=150, height=100)
    slide.text("Text W1", "monospace")


@check()
def test_text_align(deck):
    slide = deck.new_slide(width=100, height=130)
    slide.set_style("default", TextStyle(size=12))
    slide.text("Hello world\nNow!\nand there", align="start", bg_color="red")
    slide.text(
        "Hello world\nNow!\nand there",
        style=TextStyle(color="green"),
        align="center",
        bg_color="gray",
    )
    slide.text(
        "Hello world\nNow!\nand there",
        style=TextStyle(color="blue"),
        align="end",
        bg_color="#990099",
    )


@check()
def test_text_descent_ascent1(deck):
    slide = deck.new_slide(width=200, height=280)
    slide.set_style("default", TextStyle(size=24))
    box = slide.box(row=True)
    box.text("W1yg", bg_color="green", style=TextStyle(line_spacing=1.0))
    box.text("W1yg", bg_color="red", style=TextStyle(line_spacing=1.2))

    box = slide.box(row=True, bg_color="gray")
    box.text("Wy\nWy", bg_color="green", style=TextStyle(line_spacing=1.0))
    box.text("Wy\nWy", bg_color="cyan", style=TextStyle(line_spacing=1.5))

    slide.set_style("a", TextStyle(size=6))
    slide.text("Wg~a{Wg}~monospace{Wg}", bg_color="orange")

    slide.set_style("b", TextStyle(line_spacing=1.5))
    slide.text("Wg\nWg\n~b{Wg\n~a{Wg\nWg}}\nWg", bg_color="gray")


@check()
def test_text_descent_ascent2(deck):
    slide = deck.new_slide(width=20, height=380)
    slide.set_style("default", TextStyle(size=12, line_spacing=1.5))
    slide.text("g\n" * 20, bg_color="red")


@check()
def test_line_box(deck):
    deck.set_style("default", TextStyle(size=8))
    deck.set_style("big", TextStyle(size=18))
    slide = deck.new_slide(width=120, height=100)
    t = slide.text("First line\n\n~big{Third} line\n4. line\nLooooooooong line", z_level=1)

    slide.box(
        x=t.line_x(0),
        y=t.line_y(0),
        width=t.line_width(0),
        height=t.line_height(0),
        bg_color="green",
    )
    slide.box(
        x=t.line_x(1),
        y=t.line_y(1),
        width=t.line_width(1),
        height=t.line_height(1),
        bg_color="blue",
    )
    slide.box(
        x=t.line_x(2),
        y=t.line_y(2),
        width=t.line_width(2),
        height=t.line_height(2),
        bg_color="red",
    )
    slide.box(
        x=t.line_x(4),
        y=t.line_y(4),
        width=t.line_width(4),
        height=t.line_height(4),
        bg_color="orange",
    )


@check()
def test_text_line_points1(deck):
    deck.set_style("default", TextStyle(size=12))
    deck.set_style("big", TextStyle(size=18))
    slide = deck.new_slide(width=150, height=130)
    t1 = slide.text("x\nyy\nzzzzz", z_level=1, align="start")
    t2 = slide.text("x\nyy\nzzzzz", z_level=1, align="center")
    t3 = slide.text("x\nyy\nzzzzz", z_level=1, align="end")

    for t in (t1, t2, t3):
        slide.box(
            x=t.line_x(0),
            y=t.line_y(0),
            width=t.line_width(0),
            height=t.line_height(0),
            bg_color="green",
        )
        slide.box(
            x=t.line_x(1),
            y=t.line_y(1),
            width=t.line_width(1),
            height=t.line_height(1),
            bg_color="blue",
        )
        slide.box(
            x=t.line_x(2),
            y=t.line_y(2),
            width=t.line_width(2),
            height=t.line_height(2),
            bg_color="red",
        )


@check(n_slides=3)
def test_text_anchor_points(deck):
    deck.set_style("default", TextStyle(size=12))
    deck.set_style("green", TextStyle(size=16, color="green"))

    for align in ("start", "center", "end"):
        slide = deck.new_slide(width=180, height=100)
        t = slide.text(
            "Hello ~green{~1{world!}}\n~2{Full line}\n>~100{1}<<<",
            z_level=1,
            align=align,
        )
        slide.box(
            x=t.text_anchor_x(1),
            y=t.text_anchor_y(1),
            width=t.text_anchor_width(1),
            height=t.text_anchor_height(1),
            bg_color="orange",
        )
        slide.box(
            x=t.text_anchor_x(2),
            y=t.text_anchor_y(2),
            width=t.text_anchor_width(2),
            height=t.text_anchor_height(2),
            bg_color="gray",
        )
        slide.box(
            x=t.text_anchor_x(100),
            y=t.text_anchor_y(100),
            width=t.text_anchor_width(100),
            height=t.text_anchor_height(100),
            bg_color="cyan",
        )


@check()
def test_text_line_points2(deck):
    deck.set_style("default", TextStyle(size=12))
    deck.set_style("green", TextStyle(size=46, color="green"))

    slide = deck.new_slide(width=250, height=150)
    t = slide.text("Hello ~green{yWy!}\nFull line gy\nWgy", z_level=1)
    slide.box(
        x=t.line_x(0),
        y=t.line_y(0),
        width=t.line_width(0),
        height=t.line_height(0),
        bg_color="orange",
    )
    slide.box(
        x=t.line_x(1),
        y=t.line_y(1),
        width=t.line_width(1),
        height=t.line_height(1),
        bg_color="blue",
    )
    slide.box(
        x=t.line_x(2),
        y=t.line_y(2),
        width=t.line_width(2),
        height=t.line_height(2),
        bg_color="orange",
    )


@check()
def test_text_boxes(deck):
    deck.set_style("default", TextStyle(size=12))

    slide = deck.new_slide(width=50, height=50)
    t = slide.text("Line 1\nL~123{ine} 2!", z_level=1)

    t.line_box(1, bg_color="green", z_level=0)
    t.text_anchor_box(123, bg_color="orange", z_level=0)


@check()
def test_text_anchors_space_prefix(deck):
    deck.update_style("code", TextStyle(size=22))
    deck.set_style("s1", TextStyle(color="green"))
    slide = deck.new_slide(width=560, height=160)
    t = slide.text(
        """                                            ~1{C}""",
        z_level=2,
        strip=False,
    )
    t.text_anchor_box(1, bg_color="orange", z_level=1)

    t = slide.text(
        "Hello ~1{world!}",
        z_level=2,
        strip=False,
    )
    t.text_anchor_box(1, bg_color="orange", z_level=1)


@check()
def test_text_anchors_styled_space_prefix(deck):
    deck.update_style("code", TextStyle(size=22))
    deck.set_style("s1", TextStyle(size=2))
    deck.set_style("s2", TextStyle(size=44))
    slide = deck.new_slide(width=200, height=100)
    t = slide.text("""~s1{   }~s2{ }~11{C}""", z_level=2)
    t.line_box(0, bg_color="gray", z_level=0)
    t.text_anchor_box(11, bg_color="orange", z_level=1)
    t = slide.text("""~s1{ }~s2{        }~s1{ }~11{C}""", z_level=2)
    t.line_box(0, bg_color="gray", z_level=0)
    t.text_anchor_box(11, bg_color="orange", z_level=1)


@check(n_slides=3)
def test_text_in_steps(deck):
    slide = deck.new_slide(width=200, height=100)
    slide.text(InSteps(["one", "two", "three"]))


@check(n_slides=3)
def test_array_text_in_steps(deck):
    slide = deck.new_slide(width=100, height=30)
    slide.set_style("default", TextStyle(size=12))
    slide.text(["Hello ", InSteps(["world", "Nelsie", "user"]), " ", InSteps({2: "!"})])



class LayoutExpr:
    def __init__(self, inner_expr):
        self._expr = inner_expr

    def __add__(self, other):
        other = unpack_layout_expr_arg(other)
        if self._expr[0] == "sum":
            expr = self._expr + (other,)
        else:
            expr = ("sum", self._expr, other)
        return LayoutExpr(expr)

    def __sub__(self, other):
        if isinstance(other, float) or isinstance(other, int):
            return self + (-other)
        raise Exception("TODO")

    @staticmethod
    def x(node_id):
        return LayoutExpr(("x", node_id))

    @staticmethod
    def y(node_id):
        return LayoutExpr(("y", node_id))

    @staticmethod
    def width(node_id, fraction):
        return LayoutExpr(("width", node_id, fraction))

    @staticmethod
    def height(node_id, fraction):
        return LayoutExpr(("height", node_id, fraction))

    @staticmethod
    def line_x(node_id, line_idx):
        return LayoutExpr(("line_x", node_id, line_idx))

    @staticmethod
    def line_y(node_id, line_idx):
        return LayoutExpr(("line_y", node_id, line_idx))

    @staticmethod
    def line_width(node_id, line_idx, fraction):
        return LayoutExpr(("line_width", node_id, line_idx, fraction))

    @staticmethod
    def line_height(node_id, line_idx, fraction):
        return LayoutExpr(("line_height", node_id, line_idx, fraction))

    @staticmethod
    def text_anchor_x(node_id, anchor_id):
        return LayoutExpr(("anchor_x", node_id, anchor_id))

    @staticmethod
    def text_anchor_y(node_id, anchor_id):
        return LayoutExpr(("anchor_y", node_id, anchor_id))

    @staticmethod
    def text_anchor_width(node_id, anchor_id, fraction):
        return LayoutExpr(("anchor_width", node_id, anchor_id, fraction))

    @staticmethod
    def text_anchor_height(node_id, anchor_id, fraction):
        return LayoutExpr(("anchor_height", node_id, anchor_id, fraction))


def unpack_layout_expr_arg(obj):
    if isinstance(obj, LayoutExpr):
        return obj._expr
    if isinstance(obj, float) or isinstance(obj, int):
        return obj
    else:
        raise Exception("Invalid parameter for layout expression")



import argparse
import importlib
import traceback
from . import nelsie as nelsie_rs
from . import watch


def parse_args():
    parser = argparse.ArgumentParser()
    subparser = parser.add_subparsers()
    watch_parser = subparser.add_parser("watch")
    watch_parser.add_argument("source_filename")
    return parser.parse_args()


def load_py_file(filename):
    watch._WATCH_SET = set()
    importlib.machinery.SourceFileLoader("slides", filename).load_module()
    s = watch._WATCH_SET
    watch._WATCH_SET = None
    s.add(filename)
    return list(s)


def main():
    args = parse_args()
    source_filename = args.source_filename
    print("Initial build", source_filename)
    files = load_py_file(source_filename)
    print("Watching", source_filename)
    while True:
        nelsie_rs.watch(files)
        print("Building ...", source_filename)
        try:
            files = load_py_file(source_filename)
            print("... finished")
        except Exception:
            traceback.print_exc()


if __name__ == "__main__":
    main()



from nelsie import TextStyle
from nelsie.helpers.list import ListBox
from testutils import check


@check()
def test_list(deck):
    slide = deck.new_slide(width=200, height=300)
    slide.set_style("default", TextStyle(size=12))
    lst = ListBox(slide)
    lst.text("One")
    lst.text("Two1\nTwo2\nTwo3")
    lst.text("Long item three")
    lst2 = lst.list()
    lst2.text("A")
    lst2.text("BB")
    lst2.text("CCC")
    lst2 = lst.list(list_type="1")
    lst2.text("A")
    lst2.text("BB")
    lst2.text("CCC")
    lst2 = lst.list(list_type="a")
    lst2.text("A")
    lst2.text("BB")
    lst2.text("CCC")
    lst2 = lst.list(list_type="A")
    lst2.text("A")
    lst2.text("BB")
    lst2.text("CCC")
    lst3 = lst2.list()
    lst3.text("Hello!")
    lst.text("Fourth item")


@check(n_slides=4)
def test_list_steps(deck):
    slide = deck.new_slide(width=150, height=150)
    slide.set_style("default", TextStyle(size=12))
    lst = ListBox(slide)
    lst.text("One")
    lst.text(show="2+", text="Two1\nTwo2\nTwo3")
    lst.text(show="3+", text="Long item three")
    lst2 = lst.list()
    lst2.text(show="4+", text="A")


@check()
def test_list_initial_value(deck):
    slide = deck.new_slide(width=100, height=50)
    slide.set_style("default", TextStyle(size=12))
    lst = ListBox(slide, list_type="1", initial_counter_value=4)
    lst.text("One")
    lst.text("Two")
    lst.text("Three")



from testutils import check

from nelsie import TextStyle


@check(n_slides=2)
def test_code_rust_syntax_highlight(deck):
    deck.update_style("code", TextStyle(size=12))
    slide = deck.new_slide(width=300, height=150)
    slide.code(
        """
/*
  This is the main function.
*/
fn main() {
    // Print text to the console.
    println!("Hello World!");
}
""",
        language="rs",
    )
    slide = deck.new_slide(width=300, height=150)
    slide.code(
        """
/*
  This is the main function.
*/
fn main() {
    // Print text to the console.
    println!("Hello World!");
}
""",
        language="rs",
        theme="Solarized (light)",
    )


@check()
def test_code_syntax_highlight_with_styles(deck):
    deck.update_style("code", TextStyle(size=12))
    deck.set_style("s1", TextStyle(size=16, color="green"))
    deck.set_style("s2", TextStyle(size=16, color="orange"))
    slide = deck.new_slide(width=300, height=210)
    slide.code(
        """
/*
  This ~s1{is} the main function.
*/
f~s1{n} ~s1{main}() {
    // Print text to the console.
    p~s1{rintl}n!(~s1{"Hello World!"});
}

~s2{
// test1
// test2
}
""",
        language="rs",
        parse_styles=True,
    )


@check()
def test_code_syntax_highlight_anchors(deck):
    deck.update_style("code", TextStyle(size=12))
    deck.set_style("s1", TextStyle(size=16, color="green"))
    slide = deck.new_slide(width=300, height=210)
    t = slide.code(
        """
# ~11{C}omment

def ~1{main}():
    p~s1{r~100{int}(}~102{~101{"Hello} w~103{o}rld!"})

if ~2{__name__ == "__main__"}:
    main()
""",
        language="py",
        parse_styles=True,
        z_level=2,
    )
    t.text_anchor_box(1, bg_color="orange", z_level=0)
    t.text_anchor_box(2, bg_color="orange", z_level=0)

    t.text_anchor_box(100, bg_color="orange", z_level=1)
    t.text_anchor_box(101, bg_color="red", z_level=1)
    t.text_anchor_box(102, bg_color="green", z_level=0)
    t.text_anchor_box(103, bg_color="blue", z_level=1)

    t.text_anchor_box(10, bg_color="blue", z_level=1)
    t.text_anchor_box(11, bg_color="blue", z_level=1)


@check()
def test_code_style_delimiters(deck):
    slide = deck.new_slide(width=300, height=50)
    slide.set_style("big", TextStyle(size=20, color="orange"))
    slide.code(
        "print('$big<Hello> world!')",
        "Python",
        style=TextStyle(size=12),
        parse_styles=True,
        style_delimiters="$<>",
    )


@check()
def test_code_language_none(deck):
    slide = deck.new_slide(width=200, height=30)
    slide.code("print('Hello world!')", None, style=TextStyle(size=12))


@check(deck_kwargs=dict(default_code_language="Python"))
def test_code_language_default(deck):
    slide = deck.new_slide(width=200, height=30)
    slide.code("print('Hello world!')", style=TextStyle(size=12))



from setuptools import setup

setup(
    name="mkdocs-nelsie-plugin",
    version="0.0.1",
    description="MkDocs plugin for building Nelsie examples in documentation",
    author="Ada Böhm",
    author_email="ada@kreatrix.org",
    url="https://github.com/spirali/nelsie",
    packages=["mkdocs_nelsie_plugin"],
    entry_points={
        "mkdocs.plugins": [
            "nelsie = mkdocs_nelsie_plugin:NelsiePlugin",
        ]
    },
)



from testutils import check

from nelsie import Arrow, InSteps, Path, Stroke, TextStyle


@check()
def test_render_paths(deck):
    slide = deck.new_slide(width=200, height=200)

    path = (
        Path(stroke=Stroke(color="orange", width=10.0))
        .move_to(0, 0)
        .line_to(50, 50)
        .line_to(50, 100)
        .quad_to(150, 100, 150, 150)
        .cubic_to(180, 190, 150, 190, 50, 150)
    )
    slide.box(width="90%", height="90%").draw(path)


@check()
def test_render_cubic(deck):
    slide = deck.new_slide(width=200, height=200)

    path = Path(stroke=Stroke(color="red", width=5.0)).move_to(10, 10).cubic_to(10, 190, 190, 190, 190, 10)
    slide.box(width="100%", height="100%").draw(path)


@check(n_slides=3)
def test_render_path_steps(deck):
    slide = deck.new_slide(width=200, height=200)

    path1 = Path(stroke=Stroke(color="orange", width=10.0)).move_to(0, 0).line_to(50, 50).line_to(50, 100)
    path2 = Path(stroke=Stroke(color="green", width=5.0)).move_to(0, 0).line_to(75, 150)
    slide.box(width="90%", height="90%").draw(InSteps({2: path1, 4: path2}))


@check(n_slides=1)
def test_render_path_dash(deck):
    slide = deck.new_slide(width=200, height=200)

    path1 = (
        Path(stroke=Stroke(color="green", width=10.0, dash_array=[10, 5, 5, 7]))
        .move_to(0, 0)
        .line_to(100, 50)
        .line_to(50, 100)
    )
    path2 = (
        Path(stroke=Stroke(color="blue", width=5.0, dash_array=[20, 5], dash_offset=10)).move_to(0, 0).line_to(75, 150)
    )
    slide.box(width="90%", height="90%").draw([path1, path2])


@check()
def test_bg_color_opacity(deck):
    slide = deck.new_slide(width=100, height=100)
    slide.box(x=0, y=0, width=75, height=75, z_level=1, bg_color="#ff000060")
    slide.box(x=25, y=25, width=75, height=75, bg_color="#00ff0060")


@check()
def test_path_opacity(deck):
    slide = deck.new_slide(width=100, height=100)
    slide.draw(
        [
            Path(stroke=Stroke(color="#ff000060", width=30)).move_to(50, 0).line_to(50, 100),
            Path(stroke=Stroke(color="#0000ff60", width=20)).move_to(0, 50).line_to(100, 50),
        ]
    )


@check()
def test_path_box_positions(deck):
    slide = deck.new_slide(width=150, height=150)
    b1 = slide.box(width=20, height=30, bg_color="green")
    slide.box(width=30, height=40)
    b2 = slide.box(width=30, height=30, bg_color="green")

    slide.draw(
        [
            Path(stroke=Stroke(color="black", width=3))
            .move_to(b2.x(), b2.y() + 10)
            .line_to(b2.x() - 30, b2.y() + 10)
            .line_to(b2.x() - 30, b1.y(0.0))
            .line_to(b1.x(1.0), b1.y(0.5))
        ]
    )


@check()
def test_path_text_line_positions(deck):
    slide = deck.new_slide(width=160, height=200)

    text = slide.text("Hello world!\nNew line", TextStyle(size=10))

    slide.draw(
        [
            Path(stroke=Stroke(color="red", width=1)).move_to(20, 190).line_to(text.line_x(0), text.line_y(0)),
            Path(stroke=Stroke(color="green", width=1)).move_to(20, 190).line_to(text.line_x(1), text.line_y(1)),
            Path(stroke=Stroke(color="orange", width=1))
            .move_to(20, 190)
            .line_to(text.line_x(0, 1.0), text.line_y(0, 1.0)),
            Path(stroke=Stroke(color="teal", width=1))
            .move_to(20, 190)
            .line_to(text.line_x(1, 1.0), text.line_y(1, 1.0)),
        ]
    )


@check(n_slides=2)
def test_path_arrows(deck):
    arrow1 = Arrow(size=30, angle=40)
    arrow2 = Arrow(size=20, angle=10, color="red")
    arrow3 = Arrow(size=30, angle=20, stroke_width=2)
    arrow4 = Arrow(size=30, angle=45, stroke_width=2, color="blue")
    arrow5 = Arrow(size=24, angle=45, inner_point=2)
    arrow6 = Arrow(size=24, angle=45, inner_point=0.5)

    slide = deck.new_slide(width=150, height=280)
    slide.draw(
        [
            Path(stroke=Stroke(color="green", width=3), arrow_end=arrow1).move_to(20, 20).line_to(100, 50),
            Path(stroke=Stroke(color="black", width=3), arrow_end=arrow2).move_to(20, 60).line_to(100, 90),
            Path(stroke=Stroke(color="black", width=3), arrow_end=arrow3).move_to(20, 100).line_to(100, 130),
            Path(stroke=Stroke(color="black", width=3), arrow_end=arrow4).move_to(20, 140).line_to(100, 170),
            Path(stroke=Stroke(color="black", width=3), arrow_end=arrow5).move_to(20, 170).line_to(100, 210),
            Path(stroke=Stroke(color="black", width=3), arrow_end=arrow6).move_to(20, 210).line_to(100, 250),
        ]
    )
    slide = deck.new_slide(width=150, height=280)
    slide.draw(
        [
            Path(stroke=Stroke(color="green", width=3), arrow_start=arrow1).move_to(20, 20).line_to(100, 50),
            Path(stroke=Stroke(color="black", width=3), arrow_start=arrow2).move_to(20, 60).line_to(100, 90),
            Path(stroke=Stroke(color="black", width=3), arrow_start=arrow3).move_to(20, 100).line_to(100, 130),
            Path(stroke=Stroke(color="black", width=3), arrow_start=arrow4).move_to(20, 140).line_to(100, 170),
            Path(stroke=Stroke(color="black", width=3), arrow_start=arrow5).move_to(20, 170).line_to(100, 210),
            Path(stroke=Stroke(color="black", width=3), arrow_start=arrow6).move_to(20, 210).line_to(100, 250),
        ]
    )


@check()
def test_path_close(deck):
    slide = deck.new_slide(width=150, height=170)
    slide.draw(
        [
            Path(stroke=Stroke(color="green", width=3), fill_color="gray")
            .move_to(20, 20)
            .line_to(120, 50)
            .line_to(70, 150)
            .close()
        ]
    )


@check()
def test_path_oval(deck):
    slide = deck.new_slide(width=150, height=150)
    slide.draw(
        [
            Path.oval(
                25,
                25,
                125,
                125,
                stroke=Stroke(color="green", width=3),
                fill_color="gray",
            ),
            Path.oval(10, 70, 140, 90, fill_color="red"),
        ]
    )
